=begin
Copyright		2012 EnglishTiger
Name:			MT_Meshmatic
Description:	Automates/Simplifies Creating Several 2D and 3D Objects :-
				Rectangle/Box/Room/Pyramid/Cone/Circle/Cylinder/2DPolygon/3DPolygon/Sphere/Tube
				Thru it's use of User Input to Control the Size and Location of the Required Geometry
History:		1.0 (2012-02-15)	1st version
				1.1 (2012-03-23)	Switched .typename to .is_a?
=end
module MT::Plugins::MT_Toolkit
	#-------------------------------------------------------------------------------
	# Create a Rectangle(2D), Box(3D), Room(3D) or Pyramid(3D)
	def self.mt_build_box(mtshape)
		mtd0 = mtd1 = mtd2 = mtd3 = mtd4 = mtd5 = mtd6 = mtd7 = mtd8 = 0
		if mtshape == "Room"
			mtd3 = mtd4 = 288
			mtd5 = 120
		else
			mtd3 = mtd4 = mtd5 = 24
		end #if
		if mtshape == "Rectangle"
			mtui_prompts = [2, 3, 4, 41, 42]
			mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd4]
		else
			mtui_prompts = [2, 3, 4, 41, 42, 43]
			mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd4, mtd5]
		end #if
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, MTIM0 + mtshape, nil)
		return if !mtrval
		mt_start(MTUM0 + mtshape)
		if mtshape == "Rectangle"
			mtr0, mtr1, mtr2, mtr3, mtr4 = mtrval
		else
			mtr0, mtr1, mtr2, mtr3, mtr4, mtr5 = mtrval
		end #if
		mtgrp = Sketchup.active_model.entities.add_group
		# Set Up A Group as a Container for the Geometry and Give it a Unique Name
		mtgrp.name = "Merlin_" + mtshape + "_" + ("%02d" % (mt_count_nodes( mtshape, 7, 1 ) + 1)).to_s
		# Then Add the Geometry
		mtpt0 = [mtr0, mtr1, mtr2]
		mtpt1 = [mtr0 + mtr3, mtr1, mtr2]
		mtpt2 = [mtr0 + mtr3, mtr1 + mtr4, mtr2]
		mtpt3 = [mtr0, mtr1 + mtr4, mtr2]
		mtface = mtgrp.entities.add_face mtpt0, mtpt1, mtpt2, mtpt3
		case mtshape
			when "Rectangle"
				mtface.reverse! if mtface.normal.dot(Z_AXIS) < 0.0
			when "Box"
				mtr5 = -mtr5 if mtface.normal.dot(Z_AXIS) < 0.0
				mtface.pushpull mtr5
			when "Room"
				mtr5 = -mtr5 if mtface.normal.dot(Z_AXIS) < 0.0
				mtface.pushpull mtr5
				# For an IMVU Room the White (Front) Faces should be on the Inside.
				mtgrp.entities.each do |mtent|
					mtent.reverse! if mtent.is_a?(Sketchup::Face)
				end #each
			when "Pyramid"
				# For a Pyramid Add the Other 4 Faces.
				mtpt4 = [mtr0 + (mtr3/2), mtr1 + (mtr4/2), mtr2 + mtr5]
				mtpt5 = [mtr0 + (mtr3/2), mtr1 + (mtr4/2), mtr2 + mtr5]
				mtpt6 = [mtr0 + (mtr3/2), mtr1 + (mtr4/2), mtr2 + mtr5]
				mtpt7 = [mtr0 + (mtr3/2), mtr1 + (mtr4/2), mtr2 + mtr5]
				mtface = mtgrp.entities.add_face mtpt0, mtpt4, mtpt1
				mtface = mtgrp.entities.add_face mtpt1, mtpt5, mtpt2
				mtface = mtgrp.entities.add_face mtpt2, mtpt6, mtpt3
				mtface = mtgrp.entities.add_face mtpt3, mtpt7, mtpt0
		end #case
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Create a Circle(2D), Cylinder(3D) or Tube(3D)
	def self.mt_build_circle(mtshape)
		case mtshape
			when "Circle"
				mtui_prompts = [2, 3, 4, 44, 45]
				mtui_defaults = [0, 0, 0, 12, 24]
			when "Cylinder"
				mtui_prompts = [2, 3, 4, 44, 43, 45]
				mtui_defaults = [0, 0, 0, 12, 24, 24]
			else
				mtui_prompts = [2, 3, 4, 44, 43, 47, 45]
				mtui_defaults = [0, 0, 0, 12, 24, 1, 24]
		end #Case
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, MTIM0 + mtshape, nil)
		return if !mtrval
		case mtshape
			when "Circle"
				mtr0, mtr1, mtr2, mtr3, mtr6 = mtrval
			when "Cylinder"
				mtr0, mtr1, mtr2, mtr3, mtr4, mtr6 = mtrval
			else
				mtr0, mtr1, mtr2, mtr3, mtr4, mtr5, mtr6 = mtrval
		end #Case
		# Make Sure Tube Thickness is Less then It's Radius
		if mtshape == "Tube" && mtr5 >= mtr3
				UI.messagebox MTEM8, MB_OK
				return
		end #if
		mt_start(MTUM0 + mtshape)
		mtpt0 = [mtr0 + mtr3, mtr1 + mtr3, mtr2]
		# Set Up A Group as a Container for the Geometry To Be Added and Give it a Unique Name
		mtgrp = Sketchup.active_model.entities.add_group
		mtgrp.name = "Merlin_" + mtshape + "_" + ("%02d" % (mt_count_nodes( mtshape, 7, 1 ) + 1)).to_s
		# Then Add the Geometry
		mtgeom = mtgrp.entities.add_circle mtpt0, [0, 0, 1], mtr3, mtr6
		mtface = mtgrp.entities.add_face mtgeom
		if mtshape == "Circle" && mtface.normal.dot(Z_AXIS) < 0.0
			mtface.reverse!
		else
			mtr4 = -mtr4 if mtface.normal.dot(Z_AXIS) < 0.0
			mtface.pushpull mtr4
		end #if
		if mtshape == "Tube"
			mtgeom = mtgrp.entities.add_circle mtpt0, [0, 0, 1], mtr3 - mtr5, mtr6
			mtgeom[0].faces[0].pushpull mtr4
		end #if
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Create a Cone
	def self.mt_build_cone(mtshape)
		mtrval = mt_user_input([2, 3, 4, 44, 43, 45], [0, 0, 0, 12, 24, 24], MTIM0 + mtshape, nil)
		return if !mtrval
		mtr0, mtr1, mtr2, mtr3, mtr4, mtr5 = mtrval
		mt_start(MTUM0 + mtshape)
		mtpt0 = [mtr0 + mtr3, mtr1 + mtr3, mtr2]
		mtpt1 = [mtr0 + (mtr3*2), mtr1 + mtr3, mtr2]
		mtpt2 = [mtr0 + mtr3, mtr1 + mtr3, mtr2 + mtr4]
		# Set Up A Group as a Container for the Geometry To Be Added and Give it a Unique Name
		mtgrp = Sketchup.active_model.entities.add_group
		mtgrp.name = "Merlin_" + mtshape + "_" + ("%02d" % (mt_count_nodes( mtshape, 7, 1 ) + 1)).to_s
		# Then Add the Geometry
		mtface = mtgrp.entities.add_face mtpt0, mtpt1, mtpt2
		mtpath = mtgrp.entities.add_circle mtpt0, [0, 0, 1], mtr3 + 1, mtr5
		mtface.followme mtpath
		mtgrp.entities.erase_entities mtpath
		mtgrp.entities.each do |mtent|
			mtent.reverse! if mtent.is_a?(Sketchup::Face)
		end #each
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Create a PolyFace(2D) or Polygon(3D)
	def self.mt_build_polygon(mtshape)
		if mtshape == "2DPolygon"
			mtui_prompts = [2, 3, 4, 44, 46]
			mtui_defaults = [0, 0, 0, 12, 6]
		else
			mtui_prompts = [2, 3, 4, 44, 43, 46]
			mtui_defaults = [0, 0, 0, 12, 24, 6]
		end #if
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, MTIM0 + mtshape, nil)
		return if !mtrval
		if mtshape == "2DPolygon"
			mtr0, mtr1, mtr2, mtr3, mtr5 = mtrval
		else
			mtr0, mtr1, mtr2, mtr3, mtr4, mtr5 = mtrval
		end #if
		mt_start(MTUM0 + mtshape)
		mtpt0 = [mtr0 + mtr3, mtr1 + mtr3, mtr2]
		# Set Up A Group as a Container for the Geometry To Be Added and Give it a Unique Name
		mtgrp = Sketchup.active_model.entities.add_group
		mtgrp.name = "Merlin_" + mtshape + "_" + ("%02d" % (mt_count_nodes( mtshape, 7, 1 ) + 1)).to_s
		# Then Add the Geometry
		mtgeom = mtgrp.entities.add_ngon mtpt0, [0, 0, 1], mtr3, mtr5
		mtface = mtgrp.entities.add_face mtgeom
		if mtshape == "3DPolygon"
			mtr4 = -mtr4 if mtface.normal.dot(Z_AXIS) < 0.0
			mtface.pushpull mtr4
		else
			mtface.reverse! if mtface.normal.dot(Z_AXIS) < 0.0
		end #if
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Create a Sphere
	def self.mt_build_sphere(mtshape)
		mtrval = mt_user_input([2, 3, 4, 44, 45], [0, 0, 0, 12, 24], MTIM0 + mtshape, nil)
		return if !mtrval
		mt_start(MTUM0 + mtshape)
		mtr0, mtr1, mtr2, mtr3, mtr4 = mtrval
		mtpt0 = [mtr0 + mtr3, mtr1 + mtr3, mtr2 + mtr3]
		# Set Up A Group as a Container for the Geometry To Be Added and Give it a Unique Name
		mtgrp = Sketchup.active_model.entities.add_group
		mtgrp.name = "Merlin_" + mtshape + "_" + ("%02d" % (mt_count_nodes( mtshape, 7, 1 ) + 1)).to_s
		# Then Add the Geometry
		mtgeom = mtgrp.entities.add_circle mtpt0, [0, 0, 1], mtr3, mtr4
		mtface = mtgrp.entities.add_face mtgeom
		mtpath = mtgrp.entities.add_circle mtpt0, [0, 1, 0], mtr3 + 1, mtr4
		mtface.followme mtpath
		mtgrp.entities.erase_entities mtpath
 		mtgrp.entities.each do |mtent|
			mtent.reverse! if mtent.is_a?(Sketchup::Face)
		end #each
		mt_end()
	end #def
end #Module
