=begin
Copyright 		2012 EnglishTiger
Name:			MT_NodeMatic
History:		1.0 (2012-02-15)	1st Version
				1.1 (2012-03-22)	Corrected Errors in the Add Camera Routines
=end
module MT::Plugins::MT_Toolkit
	# -------- Start of Merlin's NodeMatic Routines.
	#-------------------------------------------------------------------------------
	# Routine used to add Batch of Sitting or Standing Pose Node Groups.
	def self.mt_add_pose_batch(mttype)
		mtname = "PoseGroup"
		# Set the Start Number 1 higher than the last Pose Set added
		mtd0 = mt_count_nodes("seat", 0, 0) + 1
		mtd1 = mtd2 = 24
		mtd4 = mtd6 = 6
		mtd5 = mtd7 = 48
		mtd3 = mtd8 = mtd9 = mtd10 = mtd11 = mtd12 = mtd13 = mtd14 = mtd15 = mtd16 = mtd17 = mtd18 = 0
		if mttype == "Sitting"
			mtd9 = 12
			mtd11 = -21
			mtd15 = 90
		end #if
		mtui_prompts = [0, 2, 3, 4, 12, 13, 15, 19, 22, 8, 9, 10, 11]
		mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd8, mtd9, mtd11, mtd15, mtd18, mtd4, mtd5, mtd6, mtd7]
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, mttype + MTIM3 + "(Batch)", nil)
		return if !mtrval
		mtr0, mtr1, mtr2, mtr3, mtr8, mtr9, mtr11, mtr15, mtr18, mtr4, mtr5, mtr6, mtr7 = mtrval
		mtr19 = mtr1
		mt_start(MTUM1 + mttype + MTUM4)
		mtsdef = mt_create_nodedef("mo_seat")
		mthdef = mt_create_handledef("mo_handle")
		mtcdef = mt_create_nodedef("mo_catcher")
		mtpdef = mt_create_nodedef("mo_pitcher")
		# Add the Nodes by Looping on Rows within Columns
		for mtc in (1..mtr6)
			for mtr in (1..mtr4)
				# Set Up An Outer Group with a Unique Name as a Container for the Pose Node Set
				mtidn = ("%02d" % (mtr0)).to_s
				mtpgrp = Sketchup.active_model.entities.add_group
				mtpgrp.name = mtname + mtidn
				# Then Add the Seat Node
				mtinst = mtpgrp.entities.add_instance mtsdef, [mtr1, mtr2, mtr3 + mtr8]
				mtinst.name = "seat" + mtidn + "." + mttype
				# Now the Handle Node taking any Offset from the Seat into Account
				mtinst = mtpgrp.entities.add_instance mthdef, [mtr1, mtr2, mtr3 + mtr8 + mtr9]
				mtinst.name = "Handle" + mtidn
				# Then Set Up An Inner Group as a Container for the Catcher and Pitcher Nodes
				mtcgrp = mtpgrp.entities.add_group
				mtcgrp.name = "catch-pitch"
				# Add the Catcher to the Inner Group.
				mtinst = mtcgrp.entities.add_instance mtcdef, [0, 0, 0]
				mtinst.name = "Catcher" + mtidn + ".Standing"
				# Now Add the Pitcher to the Inner Group.
				mtinst = mtcgrp.entities.add_instance mtpdef, [0, -516.mm, 0]
				mtinst.transform!(Geom::Transformation.rotation([0, -516.mm, 0],[0,0,1], 180.degrees))
				mtinst.name = "Pitcher" + mtidn + ".Standing"
				# Do any required Catcher/Pitcher Set Rotations
#				mtcgrp.transform!(Geom::Transformation.rotation([0, -258.mm, 0],[1,0,0], mtr13.degrees)) if mtr13 != 0
#				mtcgrp.transform!(Geom::Transformation.rotation([0, -258.mm, 0],[0,1,0], mtr14.degrees)) if mtr14 != 0
				mtcgrp.transform!(Geom::Transformation.rotation([0, -258.mm, 0],[0,0,1], mtr15.degrees)) if mtr15 != 0
				# Move The Catcher/Pitcher Set into Position.
				mtr20 = mtr1
				mtr21 = mtr2 + mtr11
				mtr21 = mtr2 + mtr11 + 258.mm if mtr15 != 0
				mtr22 = mtr3
				mtcgrp.transform! [mtr20, mtr21, mtr22]
				# Do any required Pose Set Rotations
#				mtpgrp.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [1, 0, 0], mtr16.degrees)) if mtr16 != 0
#				mtpgrp.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 1, 0], mtr17.degrees)) if mtr17 != 0
				mtpgrp.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 0, 1], mtr18.degrees)) if mtr18 != 0
				mtr0 += 1
				mtr1 += mtr5
			end #Row Loop
			mtr1 = mtr19
			mtr2 += mtr7
		end #Column Loop
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Routine to Add a Single Pose Node Set
	# For Custom Poses this Includes a User Supplied Pose/Stance Name
	def self.mt_add_pose(mttype)
		mtname = "PoseGroup"
		# Set the Start Number 1 higher than the last pose Set added
		mtd0 = mt_count_nodes("seat", 0, 0) + 1
		mtd1 = mtd2 = mtd3 = mtd4 = mtd5 = mtd6 = mtd7 = mtd8 = mtd9 = mtd10 = mtd11 = mtd12 = mtd13 = mtd14 = 0
		case  mttype
			when "Custom"
				mtd5 = 4
				mtd7 = -24
				mtd11 = 90
				mtd15 = "Mypose"
			when "Sitting"
				mtd5 = 12
				mtd7 = -21
				mtd11 = 90
		end #case
		if mttype == "Custom"
			mtui_prompts = [1, 2, 3, 4, 12, 13, 15, 19, 22, 23]
			mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd4, mtd5, mtd7, mtd11, mtd14, mtd15]
		else
			mtui_prompts = [1, 2, 3, 4, 12, 13, 15, 19, 22]
			mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd4, mtd5, mtd7, mtd11, mtd14]
		end #if
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, mttype + MTIM3, nil)
		return if !mtrval
		mt_start(MTUM0 + mttype + MTUM4)
		mtsdef = mt_create_nodedef("mo_seat")
		mthdef = mt_create_handledef("mo_handle")
		mtcdef = mt_create_nodedef("mo_catcher")
		mtpdef = mt_create_nodedef("mo_pitcher")
		if mttype == "Custom"
			mtr0, mtr1, mtr2, mtr3, mtr8, mtr9, mtr11, mtr15, mtr18, mtr19 = mtrval
		else
			mtr0, mtr1, mtr2, mtr3, mtr8, mtr9, mtr11, mtr15, mtr18 = mtrval
			mtr19 = mttype
		end #if
		mtidn = ("%02d" % (mtr0)).to_s
		# Set Up An Outer Group with a Unique Name as a Container for the Pose Node Set To Be Added
		mtpgrp = Sketchup.active_model.entities.add_group
		mtpgrp.name = mtname + mtidn
		# Then Add the Seat Node
		mtinst = mtpgrp.entities.add_instance mtsdef, [mtr1, mtr2, mtr3 + mtr8]
		mtinst.name = "seat" + mtidn + "." + mtr19
		# Now the Handle Node taking any Offset from the Seat into Account
		mtinst = mtpgrp.entities.add_instance mthdef, [mtr1, mtr2, mtr3 + mtr8 + mtr9]
		mtinst.name = "Handle" + mtidn
		# Then Set Up An Inner Group as a Container for the Catcher and Pitcher Nodes
		mtcgrp = mtpgrp.entities.add_group
		mtcgrp.name = "catch-pitch"
		# Add the Catcher to the Inner Group.
		mtinst = mtcgrp.entities.add_instance mtcdef, [0, 0, 0]
		mtinst.name = "Catcher" + mtidn + ".Standing"
		# Now Add the Pitcher to the Inner Group.
		mtinst = mtcgrp.entities.add_instance mtpdef, [0, -516.mm, 0]
		mtinst.transform!(Geom::Transformation.rotation([0, -516.mm, 0],[0,0,1], 180.degrees))
		mtinst.name = "Pitcher" + mtidn + ".Standing"
		# Do any required Catcher/Pitcher Set Rotations
#		mtcgrp.transform!(Geom::Transformation.rotation([0, -258.mm, 0],[1,0,0], mtr13.degrees)) if mtr13 != 0
#		mtcgrp.transform!(Geom::Transformation.rotation([0, -258.mm, 0],[0,1,0], mtr14.degrees)) if mtr14 != 0
		mtcgrp.transform!(Geom::Transformation.rotation([0, -258.mm, 0],[0,0,1], mtr15.degrees)) if mtr15 != 0
		# Move The Catcher/Pitcher Set into Position.
		mtr20 = mtr1
		mtr21 = mtr2 + mtr11
		mtr21 = mtr2 + mtr11 + 258.mm if mtr15 != 0
		mtr22 = mtr3
		mtcgrp.transform! [mtr20, mtr21, mtr22]
		# Do any required Pose Set Rotations
#		mtpgrp.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [1, 0, 0], mtr16.degrees)) if mtr16 != 0
#		mtpgrp.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 1, 0], mtr17.degrees)) if mtr17 != 0
		mtpgrp.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 0, 1], mtr18.degrees)) if mtr18 != 0
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Routine used to Add Batch of Ceiling or Floor Nodes.
	def self.mt_add_furn_batch(mttype)
		# Set the Start Number 1 higher than the last Node added
		mtd0 = mt_count_nodes(mttype, 10, 1) + 1
		if mttype == "Ceiling"
			mtd1 = mtd2 = 48
			mtd3 = 120
			mtd4 = mtd6 = 5
			mtd5 = mtd7 = 96
			mtd8 = mtd9 = mtd10 = 0
		else
			mtd1 = mtd2 = 24
			mtd3 = 0
			mtd4 = mtd6 = 10
			mtd5 = mtd7 = 48
			mtd8 = mtd9 = mtd10 = 0
		end #if
		mtui_prompts = [0, 2, 3, 4, 26, 8, 9, 10, 11]
		mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd10, mtd4, mtd5, mtd6, mtd7]
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, mttype + MTIM2 + "(Batch)", nil)
		return if !mtrval
    	mtr0, mtr1, mtr2, mtr3, mtr10, mtr4, mtr5, mtr6, mtr7 = mtrval
		mtr19 = mtr1
		mt_start(MTUM1 + mttype + MTUM3)
		mtdef = mt_create_nodedef("mo_" + mttype.downcase)
		# Add the Nodes by Looping Rows within Columns
		for mtc in (1..mtr6)
			for mtr in (1..mtr4)
				# Number, Position, and Name Our New Node
				mtinst = Sketchup.active_model.entities.add_instance mtdef, [mtr1, mtr2, mtr3]
				mtinst.name = "furniture." + mttype + "." + ("%02d" % (mtr0)).to_s
				# Rotate a Ceiling Node by 180 Degrees to turn it Upside Down
				mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 1, 0], 180.degrees)) if mttype == "Ceiling"
				# Do any required Node Rotations
#				mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [1, 0, 0], mtr8.degrees)) if mtr8 != 0
#				mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 1, 0], mtr9.degrees)) if mtr9 != 0
				mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 0, 1], mtr10.degrees)) if mtr10 != 0
				mtr0 +=  1
				mtr1 += mtr5
			end #rows loop
      		mtr1 = mtr19
			mtr2 += mtr7
      	end #columns loop
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Routine used to Add a Single Furniture Node.
	def self.mt_add_furn( mttype )
		# Set the Start Number 1 higher than the last Node added
		mtd0 = mt_count_nodes(mttype, 10, 1) + 1
		mtd1 = mtd2 = mtd3 = mtd4 = mtd5 = mtd6 = 0
  		case mttype
			when "Ceiling"
				mtd3 = 96
			when "Wall"
				mtd3 = 60
		end #case
		mtui_prompts = [1, 2, 3, 4, 26]
		mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd6]
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, mttype + MTIM2, nil)
		return if !mtrval
		mtr0, mtr1, mtr2, mtr3, mtr10 = mtrval
		mt_start(MTUM0 + mttype + MTUM2)
		mtdef = mt_create_nodedef("mo_" + mttype.downcase)
		# Number, Position and Name Our New Node
		mtinst = Sketchup.active_model.entities.add_instance mtdef, [mtr1, mtr2, mtr3]
		mtinst.name = "furniture." + mttype + "." + ("%02d" % (mtr0)).to_s
		# Rotate Wall and Ceiling Nodes to get their Z Axis pointing into the Room/Scene
		mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 1, 0], 180.degrees)) if mttype == "Ceiling"
		mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [1, 0, 0], 90.degrees)) if mttype == "Wall"
		# Do any required Node Rotations
#		mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [1, 0, 0], mtr8.degrees)) if mtr8 != 0
#		mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 1, 0], mtr9.degrees)) if mtr9 != 0
		mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 0, 1], mtr10.degrees)) if mtr10 != 0
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Routine to Add Batch of Wall Nodes.
	def self.mt_add_wall_batch(mttype)
		# Set the Start Number 1 higher than the last Node added
		mtd0 = mt_count_nodes(mttype, 10, 1) + 1
		mtd1 = 24
		mtd2 = 0
		mtd3 = 96
		mtd4 = 5
		mtd5 = 96
		mtd6 = 1
		mtd7 = mtd8 = mtd9 = mtd10 = 0
		mtui_prompts = [0, 2, 3, 4, 26, 8, 9, 52, 11]
		mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd10, mtd4, mtd5, mtd6, mtd7]
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, mttype + MTIM2 + "(Batch)", nil)
		return if !mtrval
		mtr0, mtr1, mtr2, mtr3, mtr10, mtr4, mtr5, mtr6, mtr7 = mtrval
		mtr19 = mtr1
		mtr11 = mtr2
		mtr12 = mtr3
		mtr13 = mtr10
		mtr13 -= 180 if mtr10 >= 180
		mt_start(MTUM1 + mttype + MTUM3)
		mtdef = mt_create_nodedef("mo_" + mttype.downcase)
		mtgrp = Sketchup.active_model.entities.add_group
		# Set Up A Group as a Temporary Container for the Batch of Nodes To Be Added
		# This done as it's Easier to Rotate the Nodes as a Group.
		# Add the Nodes by Looping Rows within Columns
		for mtc in (1..mtr6)
			for r in (1..mtr4)
				# Number, Position and Name Our New Node
				mtinst = mtgrp.entities.add_instance mtdef, [mtr1, mtr2, mtr3]
				mtinst.name ="furniture." + mttype + "." + ("%02d" % (mtr0)).to_s
				# Wall nodes must have their Z Axis Pointing into the Room
				mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [1, 0, 0], 90.degrees))
				# Do any required Node Rotations
#				mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [1, 0, 0], mtr8.degrees)) if mtr8 != 0
#				mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 1, 0], mtr9.degrees)) if mtr9 != 0
#				mtinst.transform!(Geom::Transformation.rotation([mtr1, mtr2, mtr3], [0, 0, 1], mtr10.degrees)) if mtr10 != 0
				mtr0 += 1
				mtr1 += mtr5
			end #rows loop
      		mtr1 = mtr19
			mtr3 += mtr7
      	end #columns loop
		# Do any required Node Rotations
		mtgrp.transform!(Geom::Transformation.rotation([mtr19, mtr11, mtr12], [0, 0, 1], mtr13.degrees)) if mtr13 != 0
		# Then Explode the Group to Leave only the Nodes
		mtrval = mtgrp.explode
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Routine to Add Batch of Wall Nodes to a Room.
	def self.mt_add_wall_room(mttype)
		# Set the Start Number 1 higher than the last Node added
		mtd0 = mt_count_nodes(mttype, 10, 1) + 1
		mtui_prompts = [0, 27, 28, 29, 30, 31, 32, 5, 7, 36, 33, 6, 7, 36, 34, 5, 7, 36, 35, 6, 7, 36]
		mtui_defaults = [mtd0, 0, 0, 0, 288, 288, 4, 36, 96, 72, 4, 36, 96, 72, 4, 36, 96, 72, 4, 36, 96, 72]
		mtrval = mt_user_input(mtui_prompts , mtui_defaults, mttype + MTIM2 + "(Room)", nil)
		return if !mtrval
		mtr0, mtr1, mtr2, mtr3, mtr4 , mtr5, mtr6, mtr7, mtr8, mtr9, mtr10, mtr11, mtr12, mtr13, mtr14, mtr15, mtr16, mtr17, mtr18, mtr19, mtr20, mtr21  = mtrval
		return if mtr6 + mtr10 + mtr14 + mtr18 == 0
		mt_start(MTUM1 + mttype + MTUM3)
		mtdef = mt_create_nodedef("mo_" + mttype.downcase)
		mtgrp = Sketchup.active_model.entities.add_group
		# Set Up A Group as a Temporary Container for the Batch of Nodes To Be Added
		# Do South Wall Nodes
		if mtr6 != 0
			mtpx = mtr1 + mtr7
			mtpy = mtr2
			mtpz = mtr3 + mtr8
			for r in (1..mtr6)
				# Number, Position and Name Our New Node
				mtinst = mtgrp.entities.add_instance mtdef, [mtpx, mtpy, mtpz]
				mtinst.name ="furniture." + mttype + "." + ("%02d" % (mtr0)).to_s
				# Wall nodes must have their Z Axis Pointing into the Room
				mtinst.transform!(Geom::Transformation.rotation([mtpx, mtpy, mtpz], [1, 0, 0], 90.degrees))
				mtinst.transform!(Geom::Transformation.rotation([mtpx, mtpy, mtpz], [0, 0, 1], 180.degrees))
				mtr0 += 1
				mtpx += mtr9
			end #for r
		end #if
		# Do West Wall Nodes
		if mtr10 != 0
			mtpx = mtr1
			mtpy = mtr2 + mtr11
			mtpz = mtr3 + mtr12
			for r in (1..mtr10)
				# Number, Position and Name Our New Node
				mtinst = mtgrp.entities.add_instance mtdef, [mtpx, mtpy, mtpz]
				mtinst.name ="furniture." + mttype + "." + ("%02d" % (mtr0)).to_s
				# Wall nodes must have their Z Axis Pointing into the Room
				mtinst.transform!(Geom::Transformation.rotation([mtpx, mtpy, mtpz], [1, 0, 0], 90.degrees))
				mtinst.transform!(Geom::Transformation.rotation([mtpx, mtpy, mtpz], [0, 0, 1], 90.degrees))
				mtr0 += 1
				mtpy += mtr13
			end #for r
		end #if
		# Do North Wall Nodes
		if mtr14 != 0
			mtpx = mtr1 + mtr15
			mtpy = mtr2 + mtr5
			mtpz = mtr3 + mtr16
			for r in (1..mtr14)
				# Number, Position and Name Our New Node
				mtinst = mtgrp.entities.add_instance mtdef, [mtpx, mtpy, mtpz]
				mtinst.name ="furniture." + mttype + "." + ("%02d" % (mtr0)).to_s
				# Wall nodes must have their Z Axis Pointing into the Room
				mtinst.transform!(Geom::Transformation.rotation([mtpx, mtpy, mtpz], [1, 0, 0], 90.degrees))
#				mtinst.transform!(Geom::Transformation.rotation([mtpx, mtpy, mtpz], [0, 0, 1], 180.degrees))
				mtr0 += 1
				mtpx += mtr17
			end #for r
		end #if
		# Do East Wall Nodes
		if mtr18 != 0
			mtpx = mtr1 + mtr4
			mtpy = mtr2 + mtr19
			mtpz = mtr3 + mtr20
			for r in (1..mtr18)
				# Number, Position and Name Our New Node
				mtinst = mtgrp.entities.add_instance mtdef, [mtpx, mtpy, mtpz]
				mtinst.name ="furniture." + mttype + "." + ("%02d" % (mtr0)).to_s
				# Wall nodes must have their Z Axis Pointing into the Room
				mtinst.transform!(Geom::Transformation.rotation([mtpx, mtpy, mtpz], [1, 0, 0], 90.degrees))
				mtinst.transform!(Geom::Transformation.rotation([mtpx, mtpy, mtpz], [0, 0, 1], -90.degrees))
				mtr0 += 1
				mtpy += mtr21
			end #for r
		end #if
		# Then Explode the Group to Leave only the Nodes
		mtrval = mtgrp.explode
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Routine to Add Omni/Spot Light Nodes
	def self.mt_add_lights (mttype)
		# Set the Node Number 1 higher than the last Light added
		mtd0 = mt_count_nodes(mttype, 0, 0) + 1
		mtd1 = mtd2 = 0
		mtd3 = 120
		mtd4 = mtd5 = 255
		mtd6 = 0
		mtd6 = 255 if mttype == "Spot"
		mtui_prompts = [1, 2, 3, 4, 37, 38, 39]
		mtui_defaults = [mtd0, mtd1, mtd2, mtd3, mtd4, mtd5, mtd6]
		mtrval = mt_user_input(mtui_prompts, mtui_defaults, mttype + " Light" + MTIM1, nil)
		return if !mtrval
		mtr0, mtr1, mtr2, mtr3, mtr4, mtr5, mtr6 = mtrval
		mt_start(MTUM0 + mttype + MTUM3)
		mtidn = ("%02d" % (mtr0)).to_s
		# Add the Users Light Color to the Materials Collection.
		mtcol = mt_check_material("mo_" + mttype.downcase + mtidn, [mtr4, mtr5, mtr6])
		# Set Up A Group as a Container for the Node Sets To Be Added
		mtgrp = Sketchup.active_model.entities.add_group
		mtgrp.name = "Light_" + mttype + mtidn
		# Now add The Light
		mtdef = mt_create_lightdef("mo_" + mttype.downcase)
		mtinst = mtgrp.entities.add_instance mtdef, [mtr1, mtr2, mtr3]
		mtinst.name = mttype + mtidn
		mtinst.material = mtcol
		# For Spot Lights we also need the Target Node
		if mttype == "Spot"
			mtdef = mt_create_circledef("mo_disc", 2)
			mtinst = mtgrp.entities.add_instance mtdef, [mtr1, mtr2, 0]
			mtinst.name = mttype + mtidn + ".Target"
			mtinst.material = mtcol
		end #if
		# If this is the 1st Light to be Added we Also Need the Dummy Parent Node
		if !mt_find_nodedef("mo_room")
			mtdef = mt_create_circledef("mo_room", 3)
			mtinst = Sketchup.active_model.entities.add_instance mtdef, [0, 0, 0]
			mtinst.name = "node.Room"
		end
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Routine to Add the Camera Node Set - N.B. There Can Only be One Camera in the Workspace
	# Other than the Message Box there is No User Interface
	def self.mt_add_camera
		mtrval = mt_find_nodedef("mo_camera")
		if mtrval
			UI.messagebox MTEM2, MB_OK
			return
		end #if
		# Check to see if they Know a Camera can only be Added to a Room
		mtrval = UI.messagebox MTCM0, MB_YESNO
		return if mtrval == 7
		mt_start(MTUM0 + "Camera Dolly")
		# Set Up A Group with a Unique Name as a Container for the Camera Nodes
		mtgrp = Sketchup.active_model.entities.add_group
		mtgrp.name = "Dolly"
		# Add the Camera Node into the Workspace
		mtdef = mt_create_cameradef("mo_camera")
		mtinst = mtgrp.entities.add_instance mtdef, [0, -120, 0]
		# The Camera Has to Be Tilted by 90 Degrees on The X Axis
		mtinst.transform!(Geom::Transformation.rotation([0, -120, 0], [1, 0, 0], 90.degrees))
		mtinst.name = "camera.01.01.root"
		# Add the Target Node into the Workspace
		mtdef = mt_create_handledef("mo_target")
		mtinst = mtgrp.entities.add_instance mtdef, [0, 0, 0]
		# The Target Has to Be Tilted by 1800 Degrees on The Z Axis
		mtinst.transform!(Geom::Transformation.rotation([0, 0, 0], [0, 0, 1], 180.degrees))
		mtinst.name = "camera.01.01.Target"
		# Finally We Raise the Group up by 48" and Tilt the Camera
		mtgrp.transform! [0, 0, 48]
		mtgrp.transform!(Geom::Transformation.rotation([0, 0, 48], [1, 0, 0], -10.degrees))
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Routine to Add a Root Node
	# N.B. There Can Only be One Root Node in the Workspace
	def self.mt_add_root(mttype)
		mtdef = mt_find_nodedef("mo_root")
		if mtdef
			UI.messagebox MTEM3, MB_OK
			return
		end #if
		mtname = "Root"
		mtname = "skeleton.Room" if mttype == "Room"
		# If it's a Room Get the SceneAmbientColor from the User and add it to the Materials Collection.
		if mttype == "Room"
			mtd1 = mtd2 = mtd3 = 222
			mtui_prompts = [37, 38, 39]
			mtui_defaults = [mtd1, mtd2, mtd3]
			mtrval = mt_user_input(mtui_prompts, mtui_defaults, "Ambient Color Parameters", nil)
			return if !mtrval
			mtr1, mtr2, mtr3 = mtrval
			mtcol = mt_check_material("SceneAmbientColor", [mtr1, mtr2, mtr3])
		end #if
		mt_start(MTUM0 + mtname + MTUM2)
		# Load the Definition and Add the Node
		mtdef = mt_create_circledef("mo_root", 4)
		mtinst = Sketchup.active_model.entities.add_instance mtdef, [0, 0, 0]
		mtinst.name = mtname
		# If it's a Room Root Node Paint it
		mtinst.material = mtcol if mttype == "Room"
		mt_end()
	end #def
	# -------- End of Merlin's NodeGen Routines.
end #module