=begin
Copyright 		2012 EnglishTiger
Name:			MT_Toolbox
Description:	Various Tools to Make Life Easier
History:		1.0 (2012-02-15)	1st version
				1.1 (2012-03-23)	Added form_mesh Routines/Functions
									Rules on What Can Be Copied or Rotated
									Transfered mt_add_comp Function from MT_Theatre
=end
# -------- Start of Merlin's Toolbox Routines.
module MT::Plugins::MT_Toolkit
	#-------------------------------------------------------------------------------
	# Add an IMVU Scaled Avatar or Skydome From Merlins Component Library
	# Used to be MT_Theatre Module
	def self.mt_add_comp(mtname)
		mtrval = mt_user_input([2, 3, 4], [0, 0, 0], MTIM7 + mtname, nil)
		return if !mtrval
		mt_start(MTUM0 + mtname)
		mtr0, mtr1, mtr2 = mtrval
		mtdef = mt_check_nodedef(mtname)
		mtinst = Sketchup.active_model.entities.add_instance mtdef, [mtr0, mtr1, mtr2]
		if mtname == "Skydome"
			mtinst.name = mtname
		else
			mtinst.name = "AV_" + mtname
		end #if
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Removes Stray Lines
	def self.mt_clean_model()
		mt_start(MTUM7)
		mtsel = Sketchup.active_model.selection
		mtsel = Sketchup.active_model.active_entities if (mtsel.length == 0)
		mterase = []
		mtsel.each do |mtent|
			if mtent.is_a?(Sketchup::Edge)
				if((mtent.faces.length==0)|| (mtent.faces.length==2 && mtent.faces[0].material==mtent.faces[1].material && mtent.faces[0].normal.parallel?(mtent.faces[1].normal)))
					mterase.push mtent
				end #inner if
			end #outer if
		end	#each
		mterase.each do |mtent|
			mtent.erase! if(mtent.deleted? == false)
		end	#each
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Adds/Moves Copies of the Selected Group/Component by the Offset Amounts
	def self.mt_move_copy( )
		mtbbox = Geom::BoundingBox.new
		mtsel = Sketchup.active_model.selection
		mtrval = mtsel.find_all {|mtent| mtent.is_a?(Sketchup::Group) or mtent.is_a?(Sketchup::ComponentInstance)}.size
		if mtrval == 0
			UI.messagebox MTEM9, MB_OK
			return
		end #if
		# Make Sure they are Not Trying to Use this Option to Add Nodes
		mtrval = mt_check_meshgrp(mtsel)
		if mtrval !=0
			UI.messagebox MTEM15, MB_OK
			return
		end #if
		mtrval = mt_user_input([48, 5, 6, 7], [1, 0, 0,0], MTIM9, nil)
		return if !mtrval
		mtr0, mtr1, mtr2, mtr3 = mtrval
		if mtr1==0 && mtr2==0 && mtr3==0
			UI.messagebox MTEM10, MB_OK
			return
		end #if
		mt_start(MTUM8)
		for mtr4 in (1..mtr0)
			mtgrp = mtsel[0].copy
			mtgrp.name = mtsel[0].name
			mttran = Geom::Transformation.translation [mtr1 * mtr4, mtr2 * mtr4, mtr3 * mtr4]
			mtgrp.transform! mttran
		end #for
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Move Model to Origin, Center, Center on Origin or a User Specified Location Works with Model or Selection
	def self.mt_move_model( where )
		mtbbox = Geom::BoundingBox.new
		mtsel = Sketchup.active_model.selection
		mtsel = Sketchup.active_model.active_entities if (mtsel.length == 0)
		mt_start(MTUM9)
		# add all geometry to a temporary bounding box
		mtsel.each do |mtent|
			mtbbox.add(mtent.bounds)
		end	#each
		case where
			when "Middle"
				mttran = Geom::Transformation.new(mtbbox.center.vector_to(ORIGIN))
			when "Origin"
				mttran = Geom::Transformation.new(mtbbox.corner(0).vector_to(ORIGIN))
			when "Center"
				mttran = Geom::Transformation.new(mtbbox.center.vector_to(ORIGIN))
				mtrval = Sketchup.active_model.entities.transform_entities(mttran, mtsel.to_a )
				mtr0 = mtbbox.depth / 2
				mttran = Geom::Transformation.translation [0, 0, mtr0]
			else
				mtrval = mt_user_input([2, 3, 4], [0, 0, 0], MTIM10, nil)
				if !mtrval
					mt_end()
					return
				end #if
				mtr0, mtr1, mtr2 = mtrval
				mttran = Geom::Transformation.new(mtbbox.corner(0).vector_to(ORIGIN))
				mtrval = Sketchup.active_model.entities.transform_entities(mttran, mtsel.to_a )
				mttran = Geom::Transformation.translation [mtr0, mtr1, mtr2]
			end #case
		mtrval = Sketchup.active_model.entities.transform_entities(mttran, mtsel.to_a )
		mt_end()
	end #def
	#-------------------------------------------------------------------------------
	# Rotate Selection on Centre Point Around Any Axis
	def self.mt_rotate_model()
		mtbbox = Geom::BoundingBox.new
		mtsel = Sketchup.active_model.selection
		mtrval = mtsel.find_all {|mtent| mtent.is_a?(Sketchup::Group) or mtent.is_a?(Sketchup::ComponentInstance)}.size
		if mtrval == 0
			UI.messagebox MTEM9, MB_OK
			return
		end #if
		mtrval = 0
		# Check to make Sure they are not trying to Rotate Camera or Root Nodes
		mtsel.each do |mtent|
			if mtent.is_a?(Sketchup::Group) or mtent.is_a?(Sketchup::ComponentInstance)
				mtrval += 1 if mtent.name[0...4] == "Root"
				mtrval += 1 if mtent.name[0...5] == "Dolly"
				mtrval += 1 if mtent.name[0...7] == "camera."
				mtrval += 1 if mtent.name[0...9] == "skeleton."
			end #if
		end #each
		if mtrval !=0
			UI.messagebox MTEM20, MB_OK
			return
		end #if
		mtrval = mt_user_input([24, 25, 26], [0, 0, 0], MTIM11, nil)
		return if !mtrval
		mtr0, mtr1, mtr2 = mtrval
#		return if mtr1.abs == 0 || mtr1.abs == 360
		mt_start(MTUM10)
		# add all geometry to a temporary bounding box
		mtsel.each  do |mtent|
			mtbbox.add(mtent.bounds)
		end #each
		mtr3 = Geom::Point3d.new(mtbbox.center)
		mtsel.each do |mtent|
			mtent.transform!(Geom::Transformation.rotation(mtr3, [1, 0, 0], mtr0.degrees)) if mtr0 != 0
			mtent.transform!(Geom::Transformation.rotation(mtr3, [0, 1, 0], mtr1.degrees)) if mtr1 != 0
			mtent.transform!(Geom::Transformation.rotation(mtr3, [0, 0, 1], mtr2.degrees)) if mtr2 != 0
		end	#each
		mt_end()
	end #def
		#-------------------------------------------------------------------------------
	# Control Routine for Count Triangles Module
	def self.mt_triangle_counter()
		mtsel = Sketchup.active_model.selection
		mtsel = Sketchup.active_model.active_entities if (mtsel.length == 0)
		UI.messagebox("Triangles: " + mt_count_triangles(mtsel).to_s)
	end #def
	#-------------------------------------------------------------------------------
	# Count Triangles in Selection
	def self.mt_count_triangles(mtents)
		mtct1 = 0
		for mte in mtents
			if mte.is_a?(Sketchup::Face)
				mtct1 += mte.edges.length - 2
			end #if
			if mte.is_a?(Sketchup::Group)
				mtct1 += mt_count_triangles(mte.entities)
			end #if
			if mte.is_a?(Sketchup::ComponentInstance)
				mtct1 += mt_count_triangles(mte.definition.entities)
			end #if
		end #for
		return mtct1
	end #def
end #module
